import typer
from typing import Optional, List
from amora.compilation import amora_model_for_path

from amora.config import settings
from amora.models import (
    list_model_files,
    is_py_model,
    AmoraModel,
    list_target_files,
)
from amora.compilation import compile_statement
from amora import materialization

app = typer.Typer(
    name="amora",
    help="Amora Data Build Tool enables engineers to transform data in their warehouses "
    "by defining schemas and writing select statements with SQLAlchemy. Amora handles turning these "
    "select statements into tables and views",
)

Models = List[str]
target_option = typer.Option(
    None,
    "--target",
    "-t",
    help="Target connection configuration as defined as an amora.target.Target",
)

models_option = typer.Option(
    [],
    "--model",
    help="A model to be compiled. This option can be passed multiple times.",
)


@app.command()
def compile(
    models: Optional[Models] = models_option,
    target: Optional[str] = target_option,
) -> None:
    """
    Generates executable SQL from model files. Compiled SQL files are written to the `./target` directory.
    """
    for model_file_path in list_model_files():
        if models and model_file_path.stem not in models:
            continue

        try:
            AmoraModel_class = amora_model_for_path(model_file_path)
        except ValueError:
            continue

        if not issubclass(AmoraModel_class, AmoraModel):  # type: ignore
            continue

        source_sql_statement = AmoraModel_class.source()
        if source_sql_statement is None:
            typer.echo(f"⏭ Skipping compilation of model `{model_file_path}`")
            continue

        target_file_path = AmoraModel_class.target_path(model_file_path)
        typer.echo(
            f"🏗 Compiling model `{model_file_path}` -> `{target_file_path}`"
        )

        content = compile_statement(source_sql_statement)
        target_file_path.write_text(content)


@app.command()
def materialize(
    models: Optional[Models] = models_option,
    target: str = target_option,
    draw_dag: bool = typer.Option(False, "--draw-dag"),
) -> None:
    """
    Executes the compiled SQL againts the current target database.
    """
    model_to_task = {}

    for target_file_path in list_target_files():
        if models and target_file_path.stem not in models:
            continue

        task = materialization.Task.for_target(target_file_path)
        model_to_task[task.model.__name__] = task

    dag = materialization.DependencyDAG.from_tasks(tasks=model_to_task.values())

    if draw_dag:
        dag.draw()

    for model in dag:
        try:
            task = model_to_task[model]
        except KeyError:
            typer.echo(f"⚠️  Skipping `{model}`")
            continue
        else:
            table = materialization.materialize(
                sql=task.sql_stmt, model=task.model
            )
            if table is None:
                continue

            typer.echo(f"✅  Created `{model}` as `{table.full_table_id}`")
            typer.echo(f"    Rows: {table.num_rows}")
            typer.echo(f"    Bytes: {table.num_bytes}")


def main():
    return app()


if __name__ == "__main__":
    main()



from setuptools import setup

setup(
    name="amora",
    version="0.1",
    description="Amora Data Build Tool",
    url="http://github.com/mundipagg/amora-data-build-tool",
    author="TREX Data",
    author_email="diogo.martins@stone.com.br",
    license="MIT",
    packages=["."],
    entry_points={
        "console_scripts": ["amora=amora.cli:main"],
    },
)



import sys
from pathlib import Path

from setuptools import setup, find_packages

if sys.version_info < (3, 8):
    print("Error: Amora does not support this version of Python.")
    print("Please upgrade to Python 3.8 or higher.")
    sys.exit(1)

here = Path(__file__)
readme = here.parent.joinpath("README.md").read_text()

setup(
    name="amora",
    version="0.0.1",
    description="Amora Data Build Tool",
    long_description=readme,
    long_description_content_type="text/markdown",
    url="http://github.com/mundipagg/amora-data-build-tool",
    author="TREX Data",
    author_email="diogo.martins@stone.com.br",
    license="MIT",
    packages=find_packages(exclude=["dbt", "assets", "tests*"]),
    entry_points={
        "console_scripts": ["amora=amora.cli:main"],
    },
    install_requires=[
        "jupyter~=1.0.0",
        "matplotlib~=3.4.2",
        "networkx~=2.6.3",
        "numpy~=1.21.1",
        "pandas~=1.3.0",
        "sqlalchemy-bigquery~=1.2.0",
        "sqlmodel~=0.0.4",
        "typer[all]~=0.4.0",
    ],
    classifiers=[
        "Development Status :: 2 - Pre-Alpha",
        "License :: OSI Approved :: MIT License",
        "Operating System :: MacOS :: MacOS X",
        "Operating System :: POSIX :: Linux",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: SQL",
    ],
    python_requires=">=3.8",
)



import inspect
import os
from importlib.util import module_from_spec, spec_from_file_location
from pathlib import Path
from typing import Optional, Protocol, runtime_checkable

import sqlparse
from amora.config import settings
from amora.models import AmoraModel, Compilable, list_target_files
from sqlalchemy_bigquery import BigQueryDialect

dialect = BigQueryDialect()


@runtime_checkable
class CompilableProtocol(Protocol):
    def source(self) -> Compilable:
        ...


def compile_statement(statement: Compilable) -> str:
    raw_sql = str(
        statement.compile(
            dialect=dialect, compile_kwargs={"literal_binds": True}
        )
    )
    formatted_sql = sqlparse.format(raw_sql, reindent=True, indent_columns=True)
    return formatted_sql


def model_path_for_target_path(path: Path) -> Path:
    return Path(
        str(path)
        .replace(settings.TARGET_PATH, settings.MODELS_PATH)
        .replace(".sql", ".py"),
    )


def target_path_for_model_path(path: Path) -> Path:
    return Path(
        str(path)
        .replace(settings.MODELS_PATH, settings.TARGET_PATH)
        .replace(".py", ".sql")
    )


def amora_model_for_path(path: Path) -> AmoraModel:
    spec = spec_from_file_location(path.stem, path)
    if spec is None:
        raise ValueError(f"Invalid path `{path}`. Not a valid Python file.")

    module = module_from_spec(spec)
    # if module is None:
    #     raise ValueError(f"Invalid path `{path}`")

    if spec.loader is None:
        raise ValueError(f"Invalid path `{path}`. Unable to load module.")

    try:
        spec.loader.exec_module(module)  # type: ignore
    except ImportError:
        raise ValueError(f"Invalid path `{path}`. Unable to load module.")

    compilables = inspect.getmembers(
        module,
        lambda x: isinstance(x, CompilableProtocol)
        and inspect.isclass(x)
        and issubclass(x, AmoraModel),  # type: ignore
    )

    classes = [class_ for _name, class_ in compilables]
    if classes:
        return classes[-1]

    raise ValueError(f"Invalid path `{path}`")


def amora_model_for_target_path(path: Path) -> AmoraModel:
    model_path = model_path_for_target_path(path)
    return amora_model_for_path(model_path)


def clean_compiled_files():
    for sql_file in list_target_files():
        os.remove(sql_file)
